@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: hfs.c,v 1.29 1998/02/04 21:59:49 rob Exp $
+ * $Id: hfs.c,v 1.10 1998/08/31 04:27:26 rob Exp $
  */
 
 # ifdef HAVE_CONFIG_H
@@ -338,6 +338,8 @@ int hfs_vstat(hfsvol *vol, hfsvolent *ent)
   ent->mddate    = d_ltime(vol->mdb.drLsMod);
   ent->bkdate    = d_ltime(vol->mdb.drVolBkUp);
 
+  ent->blessed   = vol->mdb.drFndrInfo[0];
+
   return 0;
 
 fail:
@@ -356,11 +358,22 @@ int hfs_vsetattr(hfsvol *vol, hfsvolent *ent)
   if (ent->clumpsz % vol->mdb.drAlBlkSiz != 0)
     ERROR(EINVAL, "illegal clump size");
 
-  vol->mdb.drClpSiz  = ent->clumpsz;
+  /* make sure "blessed" folder exists */
 
-  vol->mdb.drCrDate  = d_mtime(ent->crdate);
-  vol->mdb.drLsMod   = d_mtime(ent->mddate);
-  vol->mdb.drVolBkUp = d_mtime(ent->bkdate);
+  if (ent->blessed &&
+      v_getdthread(vol, ent->blessed, 0, 0) <= 0)
+    ERROR(EINVAL, "illegal blessed folder");
+
+  if (vol->flags & HFS_VOL_READONLY)
+    ERROR(EROFS, 0);
+
+  vol->mdb.drClpSiz      = ent->clumpsz;
+
+  vol->mdb.drCrDate      = d_mtime(ent->crdate);
+  vol->mdb.drLsMod       = d_mtime(ent->mddate);
+  vol->mdb.drVolBkUp     = d_mtime(ent->bkdate);
+
+  vol->mdb.drFndrInfo[0] = ent->blessed;
 
   vol->flags |= HFS_VOL_UPDATE_MDB;
 
@@ -1332,7 +1345,7 @@ int hfs_rename(hfsvol *vol, const char *srcpath, const char *dstpath)
 	strcpy(dstname, ptr);
 
       if (! moving && strcmp(srcname, dstname) == 0)
-	goto done;  /* source and destination are the same */
+	goto done;  /* source and destination are identical */
 
       if (moving || d_relstring(srcname, dstname))
 	ERROR(EEXIST, "can't use destination name");
