@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: node.c,v 1.6 1998/04/11 08:27:13 rob Exp $
+ * $Id: node.c,v 1.7 1998/09/08 18:53:06 rob Exp $
  */
 
 # ifdef HAVE_CONFIG_H
@@ -32,7 +32,14 @@
 # include "data.h"
 # include "btree.h"
 
-# define NODESPACE(n)  \
+/* total bytes used by records (NOT including record offsets) */
+
+# define NODEUSED(n)	\
+  ((n).roff[(n).nd.ndNRecs] - (n).roff[0])
+
+/* total bytes available for new records (INCLUDING record offsets) */
+
+# define NODEFREE(n)	\
   (HFS_BLOCKSZ - (n).roff[(n).nd.ndNRecs] - 2 * ((n).nd.ndNRecs + 1))
 
 /*
@@ -163,7 +170,7 @@ void compact(node *np)
       int reclen;
 
       rec    = HFS_NODEREC(*np, i);
-      reclen = np->roff[i + 1] - np->roff[i];
+      reclen = HFS_RECLEN(*np, i);
 
       if (HFS_RECKEYLEN(rec) > 0)
 	{
@@ -249,58 +256,73 @@ void n_index(const node *np, byte *record, int *reclen)
 static
 int split(node *left, byte *record, int *reclen)
 {
-  node n, *right = &n;
-  int nrecs, i;
+  btree *bt = left->bt;
+  node n, *right = &n, *side = 0;
+  int mark, i;
+
+  /* create a second node by cloning the first */
 
   *right = *left;
-  right->nd.ndBLink = left->nnum;
 
   if (n_new(right) == -1)
     goto fail;
 
-  left->nd.ndFLink = right->nnum;
-  nrecs = left->nd.ndNRecs;
+  left->nd.ndFLink  = right->nnum;
+  right->nd.ndBLink = left->nnum;
 
-  for (i = 0; i < nrecs; ++i)
+  /* divide all records evenly between the two nodes */
+
+  mark = (NODEUSED(*left) + 2 * left->nd.ndNRecs + *reclen + 2) >> 1;
+
+  if (left->rnum == -1)
     {
+      side  = left;
+      mark -= *reclen + 2;
+    }
+
+  for (i = 0; i < left->nd.ndNRecs; ++i)
+    {
+      node *np;
       byte *rec;
 
-      if (i < (nrecs >> 1))
-	rec = HFS_NODEREC(*right, i);
-      else
-	rec = HFS_NODEREC(*left, i);
+      np  = (mark > 0) ? right : left;
+      rec = HFS_NODEREC(*np, i);
+
+      mark -= HFS_RECLEN(*np, i) + 2;
 
       HFS_RECKEYLEN(rec) = 0;
+
+      if (left->rnum == i)
+	{
+	  side  = (mark > 0) ? left : right;
+	  mark -= *reclen + 2;
+	}
     }
 
   compact(left);
   compact(right);
 
-  n_search(right, record);
-  if (right->rnum == -1)
-    {
-      n_search(left, record);
-      n_insertx(left, record, *reclen);
-    }
-  else
-    n_insertx(right, record, *reclen);
+  /* insert the new record and store the modified nodes */
 
-  /* store the new/modified nodes */
+  ASSERT(side);
+
+  n_search(side, record);
+  n_insertx(side, record, *reclen);
 
   if (bt_putnode(left) == -1 ||
       bt_putnode(right) == -1)
     goto fail;
 
-  /* create an index record for the new node in the parent */
+  /* create an index record in the parent for the new node */
 
   n_index(right, record, reclen);
 
   /* update link pointers */
 
-  if (left->bt->hdr.bthLNode == left->nnum)
+  if (bt->hdr.bthLNode == left->nnum)
     {
-      left->bt->hdr.bthLNode = right->nnum;
-      left->bt->flags |= HFS_BT_UPDATE_HDR;
+      bt->hdr.bthLNode = right->nnum;
+      bt->flags |= HFS_BT_UPDATE_HDR;
     }
 
   if (right->nd.ndFLink > 0)
@@ -358,12 +380,10 @@ void n_insertx(node *np, const byte *record, int reclen)
  */
 int n_insert(node *np, byte *record, int *reclen)
 {
-  compact(np);
-
   /* check for free space */
 
   if (np->nd.ndNRecs >= HFS_MAX_NRECS ||
-      *reclen + 2 > NODESPACE(*np))
+      *reclen + 2 > NODEFREE(*np))
     return split(np, record, reclen);
 
   n_insertx(np, record, *reclen);
@@ -383,8 +403,8 @@ int join(node *left, node *right, byte *record, int *flag)
 
   /* copy records and offsets */
 
-  memcpy(HFS_NODEREC(*left, left->nd.ndNRecs), HFS_NODEREC(*right, 0),
-	 right->roff[right->nd.ndNRecs] - right->roff[0]);
+  memcpy(HFS_NODEREC(*left, left->nd.ndNRecs),
+	 HFS_NODEREC(*right, 0), NODEUSED(*right));
 
   offset = left->roff[left->nd.ndNRecs] - right->roff[0];
 
@@ -445,8 +465,7 @@ int n_delete(node *np, byte *record, int *flag)
 	goto fail;
 
       if (np->nd.ndNRecs + left.nd.ndNRecs <= HFS_MAX_NRECS &&
-	  np->roff[np->nd.ndNRecs] - np->roff[0] +
-	  2 * np->nd.ndNRecs <= NODESPACE(left))
+	  NODEUSED(*np) + 2 * np->nd.ndNRecs <= NODEFREE(left))
 	return join(&left, np, record, flag);
     }
 
